# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0

import logging
import cv2
from http.server import BaseHTTPRequestHandler, HTTPServer
import time
from urllib.parse import parse_qs, urlparse
import os

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

PORT = 8081
STREAM_IMAGE_FILE = os.environ.get("STREAM_IMAGE_FILE", "/home/nvidia/detected.jpg")


class CustomHTTPRequestHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        logger.info(f"path: {self.path}")
        parsed_path = urlparse(self.path)
        if parsed_path.path.endswith(".mjpg"):
            self.send_response(200)
            self.send_header(
                "Content-type", "multipart/x-mixed-replace; boundary=--jpgboundary"
            )
            self.end_headers()
            self.wfile.write(b"Start Streaming")
            logger.info("Start Streaming")

            while True:
                if not os.path.exists(STREAM_IMAGE_FILE):
                    time.sleep(1)
                    continue

                img = cv2.imread(STREAM_IMAGE_FILE)
                if img is None:
                    time.sleep(0.1)
                    continue

                try:
                    r, buf = cv2.imencode(".jpg", img)
                    self.wfile.write(b"--jpgboundary\r\n")
                    self.send_header("Content-type", "image/jpeg")
                    self.send_header("Content-length", str(len(buf)))
                    self.end_headers()
                    self.wfile.write(bytearray(buf))
                    self.wfile.write(b"\r\n")
                except BrokenPipeError:
                    pass
                time.sleep(0.1)


logger.info("start main")

dir_path = os.path.dirname(STREAM_IMAGE_FILE)
if not os.path.exists(dir_path):
    logger.info("create stream image path")
    os.makedirs(dir_path)

with HTTPServer(("127.0.0.1", PORT), CustomHTTPRequestHandler) as http_server:
    # 外部からアクセスする場合は、0.0.0.0で起動すると繋がります
    logger.info(f"Start server at port{PORT}")
    http_server.serve_forever()
